"""
GidAppData provides an Userdata deployment, with appdirs as backendend, but easy access to files saved there. Also provides an option to 'fake' deploy it while still developing
"""

__version__ = '0.1.8'


from gidappdata.standard_appdata import *


# log = logging.getLogger('gidappdata')
